#!/usr/bin/env python3

import argparse
import sys
import textwrap

import pyperclip
from email_validator import EmailNotValidError, validate_email

from emgen import __version__
from emgen.core import local_part


def _has_clipboard():
    try:
        orig = pyperclip.paste()
        pyperclip.copy(orig)
        return True
    except pyperclip.PyperclipException:
        return False


def main():
    has_clipboard = _has_clipboard()

    parser = argparse.ArgumentParser(
        prog="emgen",
        description="Generate random email addresses.",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "-c",
        "--clipboard",
        action="store_true",
        default=False,
        help="copy addr to clipboard (default: %(default)s)"
        if has_clipboard
        else argparse.SUPPRESS,
    )
    parser.add_argument(
        "-v", "--version", action="version", version=f"%(prog)s {__version__}"
    )

    address_description = textwrap.dedent(
        f"""\
        Email addresses are made up of a local-part, "@", and a domain name.
        The local-part will be filled randomly with LENGTH characters.

        examples:
            emgen
            {local_part()}@example.com

            emgen --domain corgi.example
            {local_part()}@corgi.example

            emgen -d dane.example --length 16
            {local_part(length=16)}@dane.example
        """
    )
    address_group = parser.add_argument_group(
        "address arguments", address_description
    )
    address_group.add_argument(
        "-d",
        "--domain",
        default="example.com",
        type=str,
        help="domain name (default: %(default)s)",
        metavar="DOMAIN",
        dest="domain",
    )
    address_group.add_argument(
        "-l",
        "--length",
        default=8,
        type=int,
        help=textwrap.dedent(
            """\
            length of the generated local-part or detail
            (default: %(default)s, range: 1 to 64)
            """
        ),
        metavar="LENGTH",
        dest="length",
    )

    subaddress_description = textwrap.dedent(
        f"""\
        If a USERNAME is supplied, the local-part will be split into two
        sub-parts - username and detail - joined by "+" or the supplied
        SEPARATOR.

        The detail will be filled randomly with LENGTH characters. However,
        the combined length of the local-part, including username, separator,
        and detail, is still limited to 64 characters.

        example:
            emgen -d gmail.example --username john.doe84
            john.doe84+{local_part()}@gmail.example
        """
    )
    subaddress_group = parser.add_argument_group(
        "subaddress arguments", subaddress_description
    )
    subaddress_group.add_argument(
        "-u",
        "--username",
        default=None,
        type=str,
        help="username to prepend to the addr (default: %(default)s)",
        metavar="USERNAME",
    )
    subaddress_group.add_argument(
        "-s",
        "--separator",
        default="+",
        type=str,
        help='separator between the username and detail (default: "%(default)s")',
        metavar="SEPARATOR",
    )

    args = parser.parse_args()

    local = local_part(args.length, args.username, args.separator)
    addr = f"{local}@{args.domain}"

    try:
        validate_email(addr)
    except EmailNotValidError as e:
        message = str(e).lower().rstrip(".")
        print("error:", message)
        sys.exit(1)

    if args.clipboard:
        if has_clipboard:
            pyperclip.copy(addr)
        else:
            warning = "warning: could not find a clipboard for your system"
            print(warning, file=sys.stderr)

    print(addr)



# -*- coding:utf-8 -*-

import secrets
from typing import Optional


def local_part(
    length: int = 8, username: Optional[str] = None, separator: str = "+"
) -> str:
    """Construct an email address local-part.

    Randomly generate a local-part using a safe subset of the allowable
    characters specified in RFC 5322.

    If a username is supplied (i.e., subaddressing is used), the random
    output will be used as the "detail" sub-part of the local-part. The
    detail sub-part will be truncated if the combined length of the username,
    separator, and detail sub-parts are greater than 64.

    Args:
        length (int, optional): Length of the generated local-part or detail
            sub-part (from 1 to 64). Defaults to 8.
        username (str, optional): The username sub-part of the local-part.
            Defaults to None.
        separator (str, optional): Separator between the username and
            detail sub-parts. Defaults to "+".

    Returns:
        str: The constructed local-part.

    See Also:
        https://tools.ietf.org/html/rfc5322#section-3.4.1
        https://tools.ietf.org/html/rfc5233#section-1
        https://www.jochentopf.com/email/address.html

    """

    def _choice(length):
        chars = "0123456789abcdefghijklmnopqrstuvwxyz"
        return "".join(secrets.choice(chars) for _ in range(length))

    if username:
        combined = len(username + separator)
        if combined + length >= 64:
            length = 64 - combined
        detail = _choice(length)
        local = f"{username}{separator}{detail}"
    else:
        local = _choice(length)

    return local



# -*- coding:utf-8 -*-

from itertools import permutations

from faker import Faker
from faker.providers import BaseProvider


class _Provider(BaseProvider):
    first_last_full = ["{{first_name}}", "{{last_name}}"]
    first_last_init = first_last_full + ["{{random_letter}}"]
    prefixes = ["_".join(perm) for perm in permutations(first_last_full)]
    prefixes += [".".join(perm) for perm in permutations(first_last_full)]
    prefixes += ["".join(perm) for perm in permutations(first_last_init, 2)]
    suffixes = ["", "{{year}}", "#", "##", "###", "####", "#####"]
    username_formats = []
    for prefix in prefixes:
        for suffixes in suffixes:
            username_formats.append(prefix + suffixes)

    safe_email_tlds = ("org", "com", "net")

    def domain(self) -> str:
        return f"example.{self.random_element(self.safe_email_tlds)}"

    def username(self) -> str:
        pattern = self.random_element(self.username_formats)
        username = self.numerify(self.generator.parse(pattern)).lower()
        return username


fake = Faker()
fake.add_provider(_Provider)


def domain() -> str:
    return fake.domain()


def username() -> str:
    return fake.username()



# -*- coding:utf-8 -*-

from pathlib import Path

from setuptools import find_packages, setup

from emgen import __version__

setup(
    name="emgen",
    version=__version__,
    description="Generate random email addresses",
    long_description=Path("./README.md").read_text(),
    long_description_content_type="text/markdown",
    author="Eric Tedor",
    author_email="eric@tedor.org",
    url="https://github.com/etedor/emgen",
    download_url="https://github.com/etedor/emgen",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: OS Independent",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Communications :: Email",
        "Topic :: Utilities",
    ],
    license="MIT",
    keywords="email",
    platforms=["any"],
    include_package_data=True,
    entry_points={"console_scripts": ["emgen=emgen.cli:main"]},
    install_requires=["pyperclip"],
)



# -*- coding:utf-8 -*-

import random

from email_validator import EmailNotValidError, validate_email

from emgen.common import fake
from emgen.core import local_part


def _is_valid(addr):
    try:
        validate_email(addr)
        return True
    except EmailNotValidError:
        return False


def test_local_part(trials: int = 99_999) -> None:
    """Test generation of random email address local-parts.

    Args:
        trials (int, optional): Number of tests to perform. Defaults to 99_999.

    """
    for _ in range(trials):
        length = random.randint(4, 64)
        username = random.choice([fake.username(), None])
        separator = random.choice("-+")
        local = local_part(
            length=length, username=username, separator=separator
        )
        domain = fake.domain()

        addr = f"{local}@{domain}"
        assert _is_valid(addr)
